@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * gcu/application.h
  *
- * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,22 +29,26 @@
 #include <map>
 #include <set>
 #include <string>
+#ifdef HAVE_GO_CONF_SYNC
+#include <goffice/app/go-conf.h>
+#endif
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkrecentmanager.h>
 #include <gcu/macros.h>
 
-using namespace std;
-
+/*!\file*/
 namespace gcu {
 
 class Document;
 class Dialog;
 
+#define GCU_CONF_DIR "gchemutils"
+
 /*!\class Application gcu/application.h
 This class is a base class for applications. It provides some basic services.
 */
-class Application: public DialogOwner
+class Application: virtual public DialogOwner
 {
 friend class Document;
 friend class Dialog;
@@ -60,7 +64,10 @@ the help_name or name parameters will be used.
 The datadir variable is used to build the full path to the help file:
 "file://"+datadir+"/gnome/help/"+name+"/"+LANG+"/"+name".xml".
 */
-	Application (string name, string datadir = DATADIR, char const *help_name = NULL, char const *icon_name = NULL);
+	Application (std::string name, std::string datadir = DATADIR, char const *help_name = NULL, char const *icon_name = NULL);
+/*!
+The destructor.
+*/
 	virtual ~Application ();
 
 /*!
@@ -71,7 +78,7 @@ be added to the help uri.
 For an application named "myapp" and a tag "mytag", the uri will be:
 helpfilename\#myapp-mytag.
 */
-	void OnHelp (string s = "");
+	void OnHelp (std::string s = "");
 /*!
 @return true if both a help browser and a help file are available, false otherwise.
 */
@@ -79,7 +86,7 @@ helpfilename\#myapp-mytag.
 /*!
 @return the name of the application. Mostly useful to set the icon name of a new window.
 */
-	string &GetName () {return Name;}
+	std::string &GetName () {return Name;}
 
 /*!
 @return a GtkWindow if any. Should be overloaded by children classes.
@@ -105,19 +112,24 @@ if they use the FileChooser.
 /*!
 @return the path to the current directory.
 */
-	char const* GetCurDir () {return CurDir;}
+	char const* GetCurDir () {return CurDir.c_str ();}
 
 /*!
 @param dir the path to the new current directory.
 */
 	void SetCurDir (char const* dir);
 
+/*!
+@param dir the path to the new current directory.
+*/
+	void SetCurDir (std::string const &dir);
+
 /*!
 @param uri the uri to display.
 
 Displays uri in the default web browser if any.
 */
-	void ShowURI (string& uri);
+	void ShowURI (std::string& uri);
 
 /*!
 @param uri the uri to the package bugs database. Default value is PACKAGE_BUGREPORT.
@@ -125,14 +137,14 @@ Displays uri in the default web browser if any.
 Opens the bugs web page in the default browser if any.
 */
 	void OnBug (char const *uri = PACKAGE_BUGREPORT)
-		{string s (uri); ShowURI (s);}
+		{std::string s (uri); ShowURI (s);}
 
 /*!
 @param uri the uri to the main web page of the program. Default value is
 "http://gchemutils.nongnu.org/".
 */
 	void OnWeb (char const *uri = "http://gchemutils.nongnu.org/")
-		{string s (uri); ShowURI (s);}
+		{std::string s (uri); ShowURI (s);}
 
 /*!
 @param MailAddress the mail adress to which a message will be sent. Defaults to the
@@ -143,14 +155,9 @@ Creates a new empty message using the default mail if any.
 	void OnMail (char const *MailAddress = "mailto:gchemutils-main@nongnu.org");
 
 /*!
-@return true if the default mail agent is known, false otherwise.
+Attempts to open the \#gchemutils channel at irc.gimp.net.
 */
-	bool HasMailAgent () {return MailAgent.length () > 0;}
-
-/*!
-@return true if the default web browser is known, false otherwise.
-*/
-	bool HasWebBrowser () {return WebBrowser.length () > 0;}
+	void OnLiveAssistance ();
 
 /*!
 @return a GtkWidget for managing pixmap resolution when saving as image. This widget is
@@ -167,7 +174,7 @@ widget is intended to be added to a GtkFileChooserDialog.
 /*!
 @return a std::map of the supported pixbuf formats. Keys are the mime type names.
 */
-	map<string, GdkPixbufFormat*> &GetSupportedPixbufFormats () {return m_SupportedPixbufFormats;}
+	std::map<std::string, GdkPixbufFormat*> &GetSupportedPixbufFormats () {return m_SupportedPixbufFormats;}
 
 /*!
 @param filename the file name.
@@ -178,7 +185,44 @@ A default extension is appended to filename if none is detected.
 @return the GdkPixbuf name associated to mime_type or NULL if the file type
 is not supported by GdkPixbuf.
 */
-	char const *GetPixbufTypeName (string& filename, char const *mime_type);
+	char const *GetPixbufTypeName (std::string& filename, char const *mime_type);
+
+/*!
+@param uri the uri of the document to load.
+@param mime_type the mime type of the document.
+@param Doc the document instance which will contain the loaded data.
+
+This method loads a document using the appropriate gcu::Loader class
+instance.
+@return true if no error occurred.
+*/
+	bool Load (std::string const &uri, const gchar *mime_type, Document* Doc);
+
+/*!
+@param uri the uri to which the document should be saved.
+@param mime_type the mime type of the document.
+@param Doc the document instance which contains the data to be saved.
+
+This method saves the document using the appropriate gcu::Loader class
+instance.
+@return true if no error occurred.
+*/
+	bool Save (std::string const &uri, const gchar *mime_type, Document* Doc);
+
+/*!
+Virtual method used to create documents. Default behavior does nothing and returns NULL.
+@return the newly created document or NULL.
+*/
+	virtual Document *CreateNewDocument () {return NULL;}
+
+#ifdef HAVE_GO_CONF_SYNC
+
+/*!
+Method used to retrieve the base configuration node.
+@return the base configuration node.
+*/
+	static GOConfNode *GetConfDir ();
+#endif
 
 protected:
 
@@ -191,22 +235,23 @@ the program. Derived class might overide this method to change this.
 
 private:
 	void AddDocument (Document *Doc) {m_Docs.insert (Doc);}
-	void RemoveDocument (Document *Doc) {m_Docs.erase (Doc); if (m_Docs.size () == 0) NoMoreDocsEvent ();}
+	void RemoveDocument (Document *Doc);
 
 private:
-	string Name;
-	string HelpName;
-	string HelpBrowser;
-	string HelpFilename;
-	char *CurDir;
-	string WebBrowser;
-	string MailAgent;
+	std::string Name;
+	std::string HelpName;
+	std::string HelpBrowser;
+	std::string HelpFilename;
+	std::string CurDir;
+#ifdef HAVE_GO_CONF_SYNC
+	static GOConfNode *m_ConfDir;
+#endif
 
 protected:
 /*!
 std::map of the supported pixbuf formats. Keys are the mime type names.
 */
-	map<string, GdkPixbufFormat*> m_SupportedPixbufFormats;	
+	std::map<std::string, GdkPixbufFormat*> m_SupportedPixbufFormats;	
 
 /*!\var m_Docs
 The currently opened documents.
@@ -214,13 +259,14 @@ The currently opened documents.
 /*!\fn GetDocs()
 @return the set of currently opened documents.
 */
-GCU_PROT_PROP (set <Document*>, Docs)
+GCU_PROT_PROP (std::set <Document*>, Docs)
 /*!\fn GetScreenResolution()
 @return the current screen resolution.
 */
 GCU_RO_PROP (unsigned, ScreenResolution)
 /*!\fn SetImageResolution(unsigned ImageResolution)
 @param ImageResolution the new image resolution.
+
 Sets the image resolution used when exporting a pixmap.
 Applications can use either the image resolution or the width and height to select
 an exported image size, but not both.
@@ -234,6 +280,7 @@ an exported image size, but not both.
 GCU_PROP (unsigned, ImageResolution)
 /*!\fn SetImageWidth(unsigned Width)
 @param Width the new image width.
+
 Sets the image width used when exporting a pixmap.
 Applications can use either the image resolution or the width and height to select
 an exported image size, but not both.
@@ -247,6 +294,7 @@ an exported image size, but not both.
 GCU_PROP (unsigned, ImageWidth)
 /*!\fn SetImageHeight(unsigned Height)
 @param Height the new image height.
+
 Sets the image height used when exporting a pixmap.
 Applications can use either the image resolution or the width and height to select
 an exported image size, but not both.
