@@ -4,7 +4,7 @@
  * Gnome Crystal
  * application.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -18,20 +18,23 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
 #include "config.h"
-#include <gtk/gtk.h>
-#include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include "application.h"
 #include "globals.h"
-#include <gcu/filechooser.h>
 #include "prefs.h"
 #include "window.h"
+#include <gcu/filechooser.h>
+#include <goffice/utils/go-image.h>
+#include <gsf/gsf-output-gio.h>
+#include <glade/glade.h>
+#include <gtk/gtk.h>
+#include <cairo-pdf.h>
+#include <cairo-ps.h>
+#include <gio/gio.h>
 #include <glib/gi18n.h>
 #include <cstring>
 
@@ -40,7 +43,7 @@ using namespace std;
 
 static unsigned short nNewDocs = 1;
 
-gcApplication::gcApplication(): Application ("gcrystal-unstable")
+gcApplication::gcApplication(): Application ("gcrystal")
 {
 }
 
@@ -53,7 +56,7 @@ gcDocument *gcApplication::OnFileNew ()
 	gcDocument* pDoc = new gcDocument (this);
 	gchar buf[32];
 	g_snprintf (buf, sizeof (buf), _("Untitled%d"), nNewDocs++);
-	pDoc->SetTitle (buf);
+	pDoc->SetLabel (buf);
 	m_Docs.push_back (pDoc);
 	new gcWindow (this, pDoc);
 	return pDoc;
@@ -102,6 +105,9 @@ void gcApplication::OnSaveAsImage ()
 	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
 	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
 		l.push_front ((*i).first.c_str ());
+	l.push_front ("image/x-eps");
+	l.push_front ("application/postscript");
+	l.push_front ("application/pdf");
 	l.push_front ("model/vrml");
 	FileChooser (this, true, l, m_pActiveDoc, _("Save as image"), GetImageSizeWidget ());
 }
@@ -136,9 +142,22 @@ enum {
 	GCRYSTAL,
 	CIF,
 	VRML,
+	PDF,
+	PS,
+	EPS,
 	PIXBUF
 };
 
+static cairo_status_t cairo_write_func (void *closure, const unsigned char *data, unsigned int length)
+{
+	gboolean result;
+	GsfOutput *output = GSF_OUTPUT (closure);
+
+	result = gsf_output_write (output, length, data);
+
+	return result ? CAIRO_STATUS_SUCCESS : CAIRO_STATUS_WRITE_ERROR;
+}
+
 bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc)
 {
 	gcDocument *Doc = static_cast<gcDocument*> (pDoc);
@@ -152,6 +171,12 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			type = CIF;
 		else if (!strcmp (mime_type, "model/vrml"))
 			type = VRML;
+		else if (!strcmp (mime_type, "image/x-eps"))
+			type = EPS;
+		else if (!strcmp (mime_type, "application/postscript"))
+			type = PS;
+		else if (!strcmp (mime_type, "application/pdf"))
+			type = PDF;
 		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type)))
 			type = PIXBUF;
 		char const *ext = NULL;
@@ -165,6 +190,15 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 		case VRML:
 			ext = ".wrl";
 			break;
+		case PDF:
+			ext = ".pdf";
+			break;
+		case PS:
+			ext = ".ps";
+			break;
+		case EPS:
+			ext = ".eps";
+			break;
 		default:
 			break;
 		}
@@ -173,20 +207,40 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			if ((i <= 0) || (strcmp (filename +i, ext)))
 				filename2 += ext;
 		}
-		GnomeVFSURI *uri = gnome_vfs_uri_new (filename2.c_str ());
-		bool err = gnome_vfs_uri_exists (uri);
-		gnome_vfs_uri_unref (uri);
+		GFile *file = g_file_new_for_uri (filename2.c_str ());
+		bool err = g_file_query_exists (file, NULL);
 		gint result = GTK_RESPONSE_YES;
 		if (err) {
-			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), filename2.c_str ());
+			char *unescaped = g_uri_unescape_string (filename2.c_str (), NULL);
+			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), unescaped);
+			g_free (unescaped);
 			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
 			gtk_window_set_icon_name (GTK_WINDOW (Box), "gcrystal");
 			result = gtk_dialog_run (Box);
 			gtk_widget_destroy (GTK_WIDGET (Box));
 			g_free (message);
-			if (result == GTK_RESPONSE_YES)
-				gnome_vfs_unlink (filename2.c_str ());
+			if (result == GTK_RESPONSE_YES) {
+				// destroy the old file if needed
+				if (err) {
+					GError *error = NULL;
+					g_file_delete (file, NULL, &error);
+					if (error) {
+						char *unescaped = g_uri_unescape_string (filename2.c_str (), NULL);
+						gchar * message = g_strdup_printf (_("Error while processing %s:\n%s"), unescaped, error->message);
+						g_free (unescaped);
+						g_error_free (error);
+						GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
+						gtk_window_set_icon_name (GTK_WINDOW (Box), "gcrystal");
+						result = gtk_dialog_run (Box);
+						gtk_widget_destroy (GTK_WIDGET (Box));
+						g_free (message);
+						g_object_unref (file);
+						return false;
+					}
+				}
+			}
 		}
+		g_object_unref (file);
 		map <string, string> options; // not used at the moment
 		if (result == GTK_RESPONSE_YES)
 			switch (type) {
@@ -212,6 +266,41 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			case PIXBUF:
 				Doc->SaveAsImage (filename2, pixbuf_type, options);
 				break;
+			default: {
+				char *fnm = go_mime_to_image_format (mime_type);
+				GOImageFormat format = go_image_get_format_from_name (fnm);
+				GError *error = NULL;
+				GsfOutput *output = gsf_output_gio_new_for_uri (filename2.c_str (), &error);
+				if (error) {
+					gchar * mess = g_strdup_printf (_("Could not create stream!\n%s"), error->message);
+					GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess);
+					g_free (mess);
+					gtk_dialog_run (GTK_DIALOG (message));
+					gtk_widget_destroy (message);
+					g_error_free (error);
+					return true;
+				}
+				cairo_surface_t *surface = NULL;
+				switch (format) {
+					case GO_IMAGE_FORMAT_EPS:
+						surface = cairo_ps_surface_create_for_stream (cairo_write_func, output, GetImageWidth (), GetImageHeight ());
+						cairo_ps_surface_set_eps (surface, TRUE);
+						break;
+					case GO_IMAGE_FORMAT_PDF:
+						surface = cairo_pdf_surface_create_for_stream (cairo_write_func, output, GetImageWidth (), GetImageHeight ());
+						break;
+					case GO_IMAGE_FORMAT_PS:
+						surface = cairo_ps_surface_create_for_stream (cairo_write_func, output, GetImageWidth (), GetImageHeight ());
+						break;
+					default:
+						return true;
+				}
+				cairo_t *cr = cairo_create (surface);
+				cairo_surface_destroy (surface);
+				Doc->GetView ()->RenderToCairo (cr, GetImageWidth (), GetImageHeight ());
+				cairo_destroy (cr);
+				break;
+			}
 			}
 	} else {
 		if (!strcmp (mime_type, "application/x-gcrystal"));
@@ -256,7 +345,8 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			list <CrystalView *> *Views = Doc->GetViews ();
 			list <CrystalView *>::iterator i, iend = Views->end ();
 			int n = 1, max = Views->size ();
-			char const *title = Doc->GetTitle ();
+			Doc->RenameViews ();
+			char const *title = Doc->GetLabel ();
 			for (i = Views->begin (); i != iend; i++) {
 				gcWindow *window = dynamic_cast <gcView*> (*i)->GetWindow ();
 				GtkWindow *w = window->GetWindow ();
